@@ -1,5 +1,6 @@
 from .common import S3Artifact
 from .dag import DAG
+from .context import Context
 from .executor import Executor, RemoteExecutor
 from .io import (AutonamedDict, IfExpression, InputArtifact, InputParameter,
                  Inputs, OutputArtifact, OutputParameter, Outputs,
@@ -12,15 +13,16 @@ from .steps import Steps
 from .task import Task
 from .utils import (copy_artifact, copy_s3, download_artifact, download_s3,
                     path_list_of_artifact, s3_config, upload_artifact,
-                    upload_s3)
+                    upload_s3, randstr)
 from .workflow import Workflow, config
 
 __all__ = ["S3Artifact", "DAG", "Executor", "RemoteExecutor", "AutonamedDict",
            "IfExpression", "InputArtifact", "InputParameter", "Inputs",
-           "OutputArtifact", "OutputParameter", "Outputs", "S3Artifact",
+           "OutputArtifact", "OutputParameter", "Outputs",
            "if_expression", "OPTemplate", "PythonScriptOPTemplate",
            "ShellOPTemplate", "Resource", "SlurmJob", "SlurmJobTemplate",
            "SlurmRemoteExecutor", "Step", "argo_len", "argo_range",
            "argo_sequence", "Steps", "Task", "copy_artifact", "copy_s3",
            "download_artifact", "download_s3", "path_list_of_artifact",
-           "s3_config", "upload_artifact", "upload_s3", "Workflow", "config"]
+           "s3_config", "upload_artifact", "upload_s3", "Workflow", "config",
+           "Context", "randstr"]
