@@ -109,9 +109,11 @@ class ArgoVar:
 
     def __getitem__(self, i):
         if isinstance(i, str):
-            return ArgoVar("jsonpath(%s, '$')['%s']" % (self.expr, i))
+            item = "jsonpath(%s, '$')['%s']" % (self.expr, i)
         else:
-            return ArgoVar("jsonpath(%s, '$')[%s]" % (self.expr, i))
+            item = "jsonpath(%s, '$')[%s]" % (self.expr, i)
+        return ArgoVar("string(%s) == %s ? %s : toJson(%s)" % (
+            item, item, item, item))
 
     def __iter__(self):
         raise TypeError("'ArgoVar' object is not iterable")
@@ -479,7 +481,6 @@ class OutputParameter(ArgoVar):
             **kwargs,
     ) -> None:
         self.value_from_path = value_from_path
-        self.value_from_parameter = value_from_parameter
         self.name = name
         self.step = step
         self.template = template
@@ -491,6 +492,7 @@ class OutputParameter(ArgoVar):
             self.default = kwargs["default"]
         if "value" in kwargs:
             self.value = kwargs["value"]
+        self.value_from_parameter = value_from_parameter
 
     def __getattr__(self, key):
         if key == "expr":
@@ -529,6 +531,26 @@ class OutputParameter(ArgoVar):
                 self.type = value.type
             if value.type is None and self.type is not None:
                 value.type = self.type
+        if key == "value_from_expression" and isinstance(value, IfExpression) \
+                and isinstance(value._then, (InputParameter, OutputParameter)):
+            if self.save_as_artifact:
+                value._then.save_as_artifact = True
+            if value._then.save_as_artifact:
+                self.save_as_artifact = True
+            if self.type is None and value._then.type is not None:
+                self.type = value._then.type
+            if value._then.type is None and self.type is not None:
+                value._then.type = self.type
+        if key == "value_from_expression" and isinstance(value, IfExpression) \
+                and isinstance(value._else, (InputParameter, OutputParameter)):
+            if self.save_as_artifact:
+                value._else.save_as_artifact = True
+            if value._else.save_as_artifact:
+                self.save_as_artifact = True
+            if self.type is None and value._else.type is not None:
+                self.type = value._else.type
+            if value._else.type is None and self.type is not None:
+                value._else.type = self.type
         return super().__setattr__(key, value)
 
     def __repr__(self):
@@ -599,7 +621,9 @@ class OutputParameter(ArgoVar):
             return V1alpha1Parameter(
                 name=self.name,
                 value_from=V1alpha1ValueFrom(
-                    parameter=str(self.value_from_parameter),
+                    # to avoid double rendering
+                    parameter=str(self.value_from_parameter).replace(
+                        "{{", "").replace("}}", ""),
                     default=default),
                 global_name=self.global_name,
                 description=description)
