@@ -17,12 +17,14 @@ from .op import OP
 from .opio import Artifact, BigParameter, Parameter
 
 try:
-    from argo.workflows.client import V1Volume, V1VolumeMount
+    from argo.workflows.client import (V1Volume, V1VolumeMount,
+                                       V1alpha1UserContainer)
 
     from ..client import V1alpha1RetryStrategy
 except Exception:
     V1Volume = object
     V1VolumeMount = object
+    V1alpha1UserContainer = object
 upload_packages = []
 
 
@@ -102,6 +104,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         requests: a dict of resource requests
         limits: a dict of resource limits
         envs: environment variables
+        init_containers: init containers before the template runs
     """
 
     def __init__(self,
@@ -132,8 +135,10 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                  limits: Dict[str, str] = None,
                  upload_dflow: bool = True,
                  envs: Dict[str, str] = None,
+                 init_containers: List[V1alpha1UserContainer] = None,
                  **kwargs,
                  ) -> None:
+        self.op_class = op_class
         op = None
         if isinstance(op_class, OP):
             op = op_class
@@ -141,22 +146,6 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         class_name = op_class.__name__
         input_sign = op_class.get_input_sign()
         output_sign = op_class.get_output_sign()
-        if slices is not None:
-            assert isinstance(slices, Slices)
-            if slices.input_artifact and not slices.sub_path:
-                input_artifact_slices = {
-                    name: slices.slices for name in slices.input_artifact}
-            if slices.input_parameter:
-                input_parameter_slices = {
-                    name: slices.slices for name in slices.input_parameter}
-            if slices.output_artifact:
-                output_artifact_slices = {}
-                for name in slices.output_artifact:
-                    output_artifact_slices[name] = slices.slices
-                    output_sign[name].archive = None  # not archive for default
-            if slices.output_parameter:
-                output_parameter_slices = {
-                    name: slices.slices for name in slices.output_parameter}
         if output_artifact_save is not None:
             for name, save in output_artifact_save.items():
                 output_sign[name].save = save
@@ -169,8 +158,8 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         super().__init__(name="%s-%s" % (class_name, "".join(random.sample(
             string.digits + string.ascii_lowercase, 5))), inputs=Inputs(),
             outputs=Outputs(), volumes=volumes, mounts=mounts,
-            requests=requests, limits=limits, envs=envs)
-        self.slices = slices
+            requests=requests, limits=limits, envs=envs,
+            init_containers=init_containers)
         if timeout is not None:
             self.timeout = "%ss" % timeout
         if retry_on_transient_error is not None:
@@ -183,18 +172,9 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         self.dflow_vars = {}
         for name, sign in input_sign.items():
             if isinstance(sign, Artifact):
-                if self.slices is not None and self.slices.sub_path and name \
-                        in self.slices.input_artifact:
-                    self.inputs.parameters["dflow_%s_sub_path" %
-                                           name] = InputParameter(value=".")
-                    self.inputs.artifacts[name] = InputArtifact(
-                        path="/tmp/inputs/artifacts/%s/{{inputs.parameters."
-                        "dflow_%s_sub_path}}" % (name, name),
-                        optional=sign.optional, type=sign.type)
-                else:
-                    self.inputs.artifacts[name] = InputArtifact(
-                        path="/tmp/inputs/artifacts/" + name,
-                        optional=sign.optional, type=sign.type)
+                self.inputs.artifacts[name] = InputArtifact(
+                    path="/tmp/inputs/artifacts/" + name,
+                    optional=sign.optional, type=sign.type)
             elif isinstance(sign, BigParameter):
                 self.inputs.parameters[name] = InputParameter(
                     save_as_artifact=True, path="/tmp/inputs/parameters/"
@@ -258,9 +238,79 @@ class PythonOPTemplate(PythonScriptOPTemplate):
             python_packages += jsonpickle.__path__
             python_packages += typeguard.__path__
 
-        script = ""
+        self.python_packages = None
         if python_packages:
             self.python_packages = set(python_packages)
+
+        self.image = image
+        self.image_pull_policy = image_pull_policy
+        if isinstance(command, str):
+            self.command = [command]
+        elif command is not None:
+            self.command = command
+        else:
+            self.command = ["python"]
+        self.init_progress = "%s/%s" % (op_class.progress_current,
+                                        op_class.progress_total)
+        self.memoize_key = memoize_key
+
+        self.op_class = op_class
+        self.input_sign = input_sign
+        self.output_sign = output_sign
+        self.op = op
+        self.input_artifact_slices = input_artifact_slices
+        self.input_parameter_slices = input_parameter_slices
+        self.output_artifact_slices = output_artifact_slices
+        self.output_parameter_slices = output_parameter_slices
+        self.slices = slices
+
+    def __setattr__(self, key, value):
+        super().__setattr__(key, value)
+        if key == "slices":
+            self.init_slices(value)
+            self.render_script()
+
+    def init_slices(self, slices):
+        if slices is not None:
+            assert isinstance(slices, Slices)
+            if slices.input_artifact and not slices.sub_path:
+                self.input_artifact_slices = {
+                    name: slices.slices for name in slices.input_artifact}
+            if slices.input_parameter:
+                self.input_parameter_slices = {
+                    name: slices.slices for name in slices.input_parameter}
+            if slices.output_artifact:
+                self.output_artifact_slices = {}
+                for name in slices.output_artifact:
+                    self.output_artifact_slices[name] = slices.slices
+                    self.outputs.artifacts[name].archive = None  # no archive
+            if slices.output_parameter:
+                self.output_parameter_slices = {
+                    name: slices.slices for name in slices.output_parameter}
+
+            if slices.sub_path:
+                for name in slices.input_artifact:
+                    self.inputs.parameters["dflow_%s_sub_path" %
+                                           name] = InputParameter(value=".")
+                    sign = self.input_sign[name]
+                    self.inputs.artifacts[name] = InputArtifact(
+                        path="/tmp/inputs/artifacts/%s/{{inputs.parameters."
+                        "dflow_%s_sub_path}}" % (name, name),
+                        optional=sign.optional, type=sign.type)
+
+    def render_script(self):
+        op_class = self.op_class
+        class_name = op_class.__name__
+        op = self.op
+        input_sign = self.input_sign
+        output_sign = self.output_sign
+        input_artifact_slices = self.input_artifact_slices
+        input_parameter_slices = self.input_parameter_slices
+        output_artifact_slices = self.output_artifact_slices
+        output_parameter_slices = self.output_parameter_slices
+
+        script = ""
+        if self.python_packages:
             self.inputs.artifacts["dflow_python_packages"] = InputArtifact(
                 path="/tmp/inputs/artifacts/dflow_python_packages")
             script += "import os, sys, json\n"
@@ -290,7 +340,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                 script += "".join(pre_lines + source_lines) + "\n"
             except Exception:
                 import cloudpickle
-                if hasattr(self, "python_packages"):
+                if self.python_packages:
                     self.python_packages.update(cloudpickle.__path__)
                 else:
                     self.python_packages = set(cloudpickle.__path__)
@@ -301,11 +351,14 @@ class PythonOPTemplate(PythonScriptOPTemplate):
 
         script += "import os, sys, traceback, jsonpickle\n"
         script += "from dflow.python import OPIO, TransientError, FatalError\n"
-        script += "from dflow.python.utils import handle_input_artifact,"\
-            " handle_input_parameter\n"
-        script += "from dflow.python.utils import handle_output_artifact,"\
-            " handle_output_parameter\n"
-        script += "from %s import %s\n\n" % (op_class.__module__, class_name)
+        script += "from dflow.python.utils import handle_input_artifact," \
+                  " handle_input_parameter\n"
+        script += "from dflow.python.utils import handle_output_artifact," \
+                  " handle_output_parameter\n"
+        if op_class.__module__ != 'dflow.python.op':
+            script += f"from {op_class.__module__} import {class_name}\n\n"
+        else:  # for class generated by decorator
+            script += op_class.script
         if op is None:
             script += "op_obj = %s()\n" % class_name
         else:
@@ -351,11 +404,12 @@ class PythonOPTemplate(PythonScriptOPTemplate):
         script += "output_sign = %s.get_output_sign()\n" % class_name
         for name, sign in output_sign.items():
             if isinstance(sign, Artifact):
-                if config["save_path_as_parameter"]:
-                    self.outputs.parameters["dflow_%s_path_list" %
-                                            name].value_from_path = \
-                        "/tmp/outputs/parameters/dflow_%s_path_list" % name
                 slices = self.get_slices(output_artifact_slices, name)
+                if config["save_path_as_parameter"] or slices is not None:
+                    self.outputs.parameters["dflow_%s_path_list" %
+                                            name] = OutputParameter(
+                        value_from_path="/tmp/outputs/parameters/"
+                        "dflow_%s_path_list" % name)
                 script += "handle_output_artifact('%s', output['%s'], "\
                     "output_sign['%s'], %s, '/tmp')\n" % (name, name, name,
                                                           slices)
@@ -365,19 +419,7 @@ class PythonOPTemplate(PythonScriptOPTemplate):
                     "output_sign['%s'], %s, '/tmp')\n" % (name, name, name,
                                                           slices)
 
-        self.image = image
-        self.image_pull_policy = image_pull_policy
-        if isinstance(command, str):
-            self.command = [command]
-        elif command is not None:
-            self.command = command
-        else:
-            self.command = ["python"]
         self.script = script
-        self.init_progress = "%s/%s" % (op_class.progress_current,
-                                        op_class.progress_total)
-
-        self.memoize_key = memoize_key
 
     def get_slices(self, slices_dict, name):
         slices = None
