@@ -7,6 +7,7 @@ import jsonpickle
 
 from .common import S3Artifact
 from .config import config
+from .context_syntax import GLOBAL_CONTEXT
 from .executor import Executor
 from .io import (PVC, ArgoVar, InputArtifact, InputParameter, OutputArtifact,
                  OutputParameter)
@@ -350,6 +351,14 @@ class Step:
                     self.prepare_step.outputs.artifacts[name]
             self.template = new_template
 
+        if GLOBAL_CONTEXT.in_context:
+            if not self.name.endswith('init-artifact'):
+                GLOBAL_CONTEXT.current_workflow.add(self)
+            else:
+                if self.name.endswith('init-artifact-init-artifact'):
+                    raise ValueError(
+                        "Please don't name step as '***init-artifact'")
+
     def __repr__(self):
         return self.id
 
@@ -375,7 +384,7 @@ class Step:
                     InputParameter, OutputParameter, InputArtifact,
                     OutputArtifact)):
                 self.inputs.parameters[k].source = v
-                return
+                continue
 
             if v is None:
                 self.inputs.parameters[k].value = "null"
