@@ -9,6 +9,7 @@ from .argo_objects import ArgoStep, ArgoWorkflow
 from .common import S3Artifact
 from .config import config
 from .context import Context
+from .context_syntax import GLOBAL_CONTEXT
 from .dag import DAG
 from .step import Step
 from .steps import Steps
@@ -19,7 +20,7 @@ try:
     import kubernetes
     import urllib3
     urllib3.disable_warnings()
-    from argo.workflows.client import (ApiClient, Configuration,
+    from argo.workflows.client import (ApiClient, Configuration, V1alpha1PodGC,
                                        V1alpha1Workflow,
                                        V1alpha1WorkflowCreateRequest,
                                        V1alpha1WorkflowSpec, V1ObjectMeta,
@@ -53,6 +54,14 @@ class Workflow:
         context: context for the workflow
         annotations: annotations for the workflow
         parallelism: maximum number of running pods for the workflow
+        pod_gc_stategy: pod GC provides the ability to delete pods
+            automatically without deleting the workflow, pod GC strategy
+            must be one of the following:
+            * OnPodCompletion - delete pods immediately when pod is completed
+                (including errors/failures)
+            * OnPodSuccess - delete pods immediately when pod is successful
+            * OnWorkflowCompletion - delete pods when workflow is completed
+            * OnWorkflowSuccess - delete pods when workflow is successful
     """
 
     def __init__(
@@ -68,7 +77,8 @@ class Workflow:
             k8s_api_server: str = None,
             context: Context = None,
             annotations: Dict[str, str] = None,
-            parallelism: int = None
+            parallelism: int = None,
+            pod_gc_strategy: str = None,
     ) -> None:
         self.host = host if host is not None else config["host"]
         self.token = token if token is not None else config["token"]
@@ -81,6 +91,7 @@ class Workflow:
             annotations = {}
         self.annotations = annotations
         self.parallelism = parallelism
+        self.pod_gc_strategy = pod_gc_strategy
 
         configuration = Configuration(host=self.host)
         configuration.verify_ssl = False
@@ -111,6 +122,14 @@ class Workflow:
             self.pvcs = {}
             self.id = None
 
+    def __enter__(self) -> 'Workflow':
+        GLOBAL_CONTEXT.in_context = True
+        GLOBAL_CONTEXT.current_workflow = self
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
+        GLOBAL_CONTEXT.in_context = False
+
     def add(
             self,
             step: Union[Step, List[Step], Task, List[Task]],
@@ -258,7 +277,9 @@ class Workflow:
                 entrypoint=self.entrypoint.name,
                 templates=list(self.argo_templates.values()),
                 parallelism=self.parallelism,
-                volume_claim_templates=argo_pvcs),
+                volume_claim_templates=argo_pvcs,
+                pod_gc=V1alpha1PodGC(strategy=self.pod_gc_strategy),
+            ),
             status=status)
 
     def handle_template(self, template, memoize_prefix=None,
@@ -354,7 +375,22 @@ class Workflow:
         Returns:
             a list of keys
         """
-        return [step.key for step in self.query_step() if step.key is not None]
+        try:
+            try:
+                response = self.api_instance.api_client.call_api(
+                    '/api/v1/workflows/%s/%s' % (self.namespace, self.id),
+                    'GET', response_type=object, _return_http_data_only=True,
+                    query_params=[('fields', 'status.outputs')])
+            except Exception:
+                response = self.api_instance.api_client.call_api(
+                    '/api/v1/archived-workflows/%s' % self.id,
+                    'GET', response_type=object, _return_http_data_only=True,
+                    query_params=[('fields', 'status.outputs')])
+            return [par["name"] for par in
+                    response["status"]["outputs"]["parameters"]]
+        except Exception:
+            return [step.key for step in self.query_step()
+                    if step.key is not None]
 
     def terminate(self) -> None:
         """
